@@ -40,7 +40,12 @@ orc_executor_run (OrcExecutor *ex)
 {
   void (*func) (OrcExecutor *);
 
-  func = ex->program->code_exec;
+  if (ex->program) {
+    func = ex->program->code_exec;
+  } else {
+    OrcCode *code = (OrcCode *)ex->arrays[ORC_VAR_A2];
+    func = code->exec;
+  }
   if (func) {
     func (ex);
     //ORC_ERROR("counters %d %d %d", ex->counter1, ex->counter2, ex->counter3);
@@ -54,7 +59,12 @@ orc_executor_run_backup (OrcExecutor *ex)
 {
   void (*func) (OrcExecutor *);
 
-  func = ex->program->backup_func;
+  if (ex->program) {
+    func = ex->program->backup_func;
+  } else {
+    OrcCode *code = (OrcCode *)ex->arrays[ORC_VAR_A2];
+    func = code->exec;
+  }
   if (func) {
     func (ex);
     //ORC_ERROR("counters %d %d %d", ex->counter1, ex->counter2, ex->counter3);
@@ -214,12 +224,18 @@ orc_executor_emulate (OrcExecutor *ex)
   int j;
   int k;
   int m, m_index;
-  OrcCode *code = ex->program->orccode;
+  OrcCode *code;
   OrcInstruction *insn;
   OrcStaticOpcode *opcode;
   OrcOpcodeExecutor *opcode_ex;
   void *tmpspace[ORC_N_COMPILER_VARIABLES] = { 0 };
 
+  if (ex->program) {
+    code = ex->program->orccode;
+  } else {
+    code = (OrcCode *)ex->arrays[ORC_VAR_A2];
+  }
+
   ex->accumulators[0] = 0;
   ex->accumulators[1] = 0;
   ex->accumulators[2] = 0;
@@ -281,13 +297,13 @@ orc_executor_emulate (OrcExecutor *ex)
       } else if (var->vartype == ORC_VAR_TYPE_TEMP) {
         opcode_ex[j].src_ptrs[k] = tmpspace[insn->src_args[k]];
       } else if (var->vartype == ORC_VAR_TYPE_SRC) {
-        if (((unsigned long)ex->arrays[insn->src_args[k]]) & (var->size - 1)) {
+        if (ORC_PTR_TO_INT(ex->arrays[insn->src_args[k]]) & (var->size - 1)) {
           ORC_ERROR("Unaligned array for src%d, program %s",
               (insn->src_args[k]-ORC_VAR_S1), ex->program->name);
         }
         opcode_ex[j].src_ptrs[k] = ex->arrays[insn->src_args[k]];
       } else if (var->vartype == ORC_VAR_TYPE_DEST) {
-        if (((unsigned long)ex->arrays[insn->src_args[k]]) & (var->size - 1)) {
+        if (ORC_PTR_TO_INT(ex->arrays[insn->src_args[k]]) & (var->size - 1)) {
           ORC_ERROR("Unaligned array for dest%d, program %s",
               (insn->src_args[k]-ORC_VAR_D1), ex->program->name);
         }
@@ -305,7 +321,7 @@ orc_executor_emulate (OrcExecutor *ex)
         opcode_ex[j].dest_ptrs[k] =
           &ex->accumulators[insn->dest_args[k] - ORC_VAR_A1];
       } else if (var->vartype == ORC_VAR_TYPE_DEST) {
-        if (((unsigned long)ex->arrays[insn->dest_args[k]]) & (var->size - 1)) {
+        if (ORC_PTR_TO_INT(ex->arrays[insn->dest_args[k]]) & (var->size - 1)) {
           ORC_ERROR("Unaligned array for dest%d, program %s",
               (insn->dest_args[k]-ORC_VAR_D1), ex->program->name);
         }
