@@ -52,6 +52,7 @@ orc_target_c_get_typedefs (void)
     "typedef unsigned __int32 orc_uint32;\n"
     "typedef unsigned __int64 orc_uint64;\n"
     "#define ORC_UINT64_C(x) (x##Ui64)\n"
+    "#define inline __inline\n"
     "#else\n"
     "#include <limits.h>\n"
     "typedef signed char orc_int8;\n"
@@ -73,6 +74,15 @@ orc_target_c_get_typedefs (void)
     "typedef union { orc_int16 i; orc_int8 x2[2]; } orc_union16;\n"
     "typedef union { orc_int32 i; float f; orc_int16 x2[2]; orc_int8 x4[4]; } orc_union32;\n"
     "typedef union { orc_int64 i; double f; orc_int32 x2[2]; float x2f[2]; orc_int16 x4[4]; } orc_union64;\n"
+    "#endif\n"
+    "#ifndef ORC_RESTRICT\n"
+    "#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L\n"
+    "#define ORC_RESTRICT restrict\n"
+    "#elif defined(__GNUC__) && __GNUC__ >= 4\n"
+    "#define ORC_RESTRICT __restrict__\n"
+    "#else\n"
+    "#define ORC_RESTRICT\n"
+    "#endif\n"
     "#endif\n";
 }
 
@@ -111,6 +121,7 @@ orc_target_c_get_asm_preamble (void)
     "#define ORC_ISNAN(x) ((((x)&0x7f800000) == 0x7f800000) && (((x)&0x007fffff) != 0))\n"
     "#define ORC_DENORMAL_DOUBLE(x) ((x) & ((((x)&ORC_UINT64_C(0x7ff0000000000000)) == 0) ? ORC_UINT64_C(0xfff0000000000000) : ORC_UINT64_C(0xffffffffffffffff)))\n"
     "#define ORC_ISNAN_DOUBLE(x) ((((x)&ORC_UINT64_C(0x7ff0000000000000)) == ORC_UINT64_C(0x7ff0000000000000)) && (((x)&ORC_UINT64_C(0x000fffffffffffff)) != 0))\n"
+    "#ifndef ORC_RESTRICT\n"
     "#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L\n"
     "#define ORC_RESTRICT restrict\n"
     "#elif defined(__GNUC__) && __GNUC__ >= 4\n"
@@ -118,6 +129,7 @@ orc_target_c_get_asm_preamble (void)
     "#else\n"
     "#define ORC_RESTRICT\n"
     "#endif\n"
+    "#endif\n"
     "/* end Orc C target preamble */\n\n";
 }
 
@@ -318,7 +330,7 @@ orc_compiler_c_assemble (OrcCompiler *compiler)
     if (!rule) {
       ORC_COMPILER_ERROR(compiler, "No rule for: %s on target %s", opcode->name,
           compiler->target->name);
-      compiler->error = TRUE;
+      continue;
     }
     ORC_ASM_CODE(compiler,"%*s", prefix, "");
     if (insn->flags & (ORC_INSTRUCTION_FLAG_X2|ORC_INSTRUCTION_FLAG_X4)) {
@@ -356,7 +368,7 @@ orc_compiler_c_assemble (OrcCompiler *compiler)
     if (!rule) {
       ORC_COMPILER_ERROR(compiler, "No rule for: %s on target %s", opcode->name,
           compiler->target->name);
-      compiler->error = TRUE;
+      continue;
     }
 
     if (insn->flags & (ORC_INSTRUCTION_FLAG_X2|ORC_INSTRUCTION_FLAG_X4)) {
@@ -784,7 +796,7 @@ c_rule_loadpX (OrcCompiler *p, void *user, OrcInstruction *insn)
     }
   } else if (p->vars[insn->src_args[0]].vartype == ORC_VAR_TYPE_CONST) {
     if (p->vars[insn->src_args[0]].size <= 4) {
-      ORC_ASM_CODE(p,"    %s = 0x%08x; /* %d or %gf */\n", dest,
+      ORC_ASM_CODE(p,"    %s = (int)0x%08x; /* %d or %gf */\n", dest,
           (unsigned int)p->vars[insn->src_args[0]].value.i,
           (int)p->vars[insn->src_args[0]].value.i,
           p->vars[insn->src_args[0]].value.f);
@@ -1078,7 +1090,7 @@ c_rule_div255w (OrcCompiler *p, void *user, OrcInstruction *insn)
   c_get_name_int (src, p, insn, insn->src_args[0]);
 
   ORC_ASM_CODE(p,
-      "    %s = ((uint16_t)(((orc_uint16)(%s+128)) + (((orc_uint16)(%s+128))>>8)))>>8;\n",
+      "    %s = ((orc_uint16)(((orc_uint16)(%s+128)) + (((orc_uint16)(%s+128))>>8)))>>8;\n",
       dest, src, src);
 }
 
@@ -1092,7 +1104,7 @@ c_rule_divluw (OrcCompiler *p, void *user, OrcInstruction *insn)
   c_get_name_int (src2, p, insn, insn->src_args[1]);
 
   ORC_ASM_CODE(p,
-      "    %s = ((%s&0xff) == 0) ? 255 : ORC_CLAMP_UB(((uint16_t)%s)/((uint16_t)%s&0xff));\n",
+      "    %s = ((%s&0xff) == 0) ? 255 : ORC_CLAMP_UB(((orc_uint16)%s)/((orc_uint16)%s&0xff));\n",
       dest, src2, src1, src2);
 }
 
