@@ -4,6 +4,8 @@
 
 #include <orc/orcutils.h>
 
+ORC_BEGIN_DECLS
+
 typedef struct _OrcOpcodeExecutor OrcOpcodeExecutor;
 typedef struct _OrcExecutor OrcExecutor;
 typedef struct _OrcExecutorAlt OrcExecutorAlt;
@@ -50,9 +52,6 @@ typedef void (*OrcExecutorFunc)(OrcExecutor *ex);
 
 #define ORC_MAX_VAR_SIZE 8
 
-#define ORC_STRUCT_OFFSET(struct_type, member)    \
-      ((long) ((unsigned int *) &((struct_type*) 0)->member))
-
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -83,6 +82,7 @@ enum {
   ORC_TARGET_C_BARE = (1<<1),
   ORC_TARGET_C_NOEXEC = (1<<2),
   ORC_TARGET_C_OPCODE = (1<<3),
+  ORC_TARGET_CLEAN_COMPILE = (1<<29),
   ORC_TARGET_FAST_NAN = (1<<30),
   ORC_TARGET_FAST_DENORMAL = (1<<31)
 };
@@ -98,7 +98,8 @@ enum {
 };
 
 enum {
-  ORC_TARGET_ARM_EDSP = (1<<2)
+  ORC_TARGET_ARM_EDSP = (1<<2),
+  ORC_TARGET_ARM_ARM6 = (1<<3)
 };
 
 typedef enum {
@@ -231,6 +232,8 @@ struct _OrcVariable {
   int aligned_data;
   int param_type;
   int load_dest;
+  int update_type;
+  int need_offset_reg;
 };
 
 /**
@@ -325,6 +328,7 @@ struct _OrcConstant {
   unsigned int value;
   unsigned int full_value[4];
   int use_count;
+  int is_long;
 };
 
 /**
@@ -458,6 +462,7 @@ struct _OrcCompiler {
 
   int is_64bit;
   int tmpreg;
+  int tmpreg2;
   int exec_reg;
   int gp_tmpreg;
 
@@ -477,6 +482,7 @@ struct _OrcCompiler {
   int max_used_temp_reg;
 
   int insn_shift; /* used when emitting rules */
+  int max_var_size; /* size of largest var */
 };
 
 #define ORC_SRC_ARG(p,i,n) ((p)->vars[(i)->src_args[(n)]].alloc)
@@ -519,6 +525,7 @@ struct _OrcExecutor {
   int params[ORC_N_VARIABLES];
   int accumulators[4];
   /* exec pointer is stored in arrays[ORC_VAR_A1] */
+  /* OrcCode pointer is stored in arrays[ORC_VAR_A2] */
   /* row pointers are stored in arrays[i+ORC_VAR_C1] */
   /* the stride for arrays[x] is stored in params[x] */
   /* m is stored in params[ORC_VAR_A1] */
@@ -538,7 +545,8 @@ struct _OrcExecutorAlt {
 
   void *arrays[ORC_N_ARRAYS];
   OrcExecutorFunc exec;
-  void *unused1[ORC_N_VARIABLES - ORC_N_ARRAYS - 1];
+  OrcCode *code;
+  void *unused1[ORC_N_VARIABLES - ORC_N_ARRAYS - 2];
   int strides[ORC_N_ARRAYS];
   int m;
   int m_index;
@@ -564,12 +572,14 @@ struct _OrcCodeVariable {
 };
 
 struct _OrcCode {
+  /*< public >*/
+  OrcExecutorFunc exec;
+
   /*< private >*/
   OrcCompileResult result;
   char *name;
 
   /* for execution */
-  OrcExecutorFunc exec;
   unsigned char *code;
   int code_size;
   void *chunk;
@@ -603,8 +613,10 @@ struct _OrcTarget {
   void (*load_constant)(OrcCompiler *compiler, int reg, int size, int value);
   const char * (*get_flag_name)(int shift);
   void (*flush_cache) (OrcCode *code);
+  void (*load_constant_long)(OrcCompiler *compiler, int reg,
+      OrcConstant *constant);
 
-  void *_unused[6];
+  void *_unused[5];
 };
 
 
@@ -714,6 +726,10 @@ int orc_program_allocate_register (OrcProgram *program, int is_data);
 void orc_code_allocate_codemem (OrcCode *code, int size);
 int orc_compiler_label_new (OrcCompiler *compiler);
 int orc_compiler_get_constant (OrcCompiler *compiler, int size, int value);
+int orc_compiler_get_constant_long (OrcCompiler *compiler, orc_uint32 a,
+    orc_uint32 b, orc_uint32 c, orc_uint32 d);
+int orc_compiler_try_get_constant_long (OrcCompiler *compiler, orc_uint32 a,
+    orc_uint32 b, orc_uint32 c, orc_uint32 d);
 int orc_compiler_get_temp_constant (OrcCompiler *compiler, int size, int value);
 int orc_compiler_get_temp_reg (OrcCompiler *compiler);
 int orc_compiler_get_constant_reg (OrcCompiler *compiler);
@@ -731,7 +747,6 @@ void orc_compiler_append_code (OrcCompiler *p, const char *fmt, ...)
  
 void orc_target_register (OrcTarget *target);
 OrcTarget *orc_target_get_by_name (const char *target_name);
-int orc_program_get_max_var_size (OrcProgram *program);
 int orc_program_get_max_array_size (OrcProgram *program);
 int orc_program_get_max_accumulator_size (OrcProgram *program);
 
@@ -756,11 +771,14 @@ extern int _orc_cpu_stepping;
 extern const char *_orc_cpu_name;
 
 extern int _orc_compiler_flag_backup;
+extern int _orc_compiler_flag_emulate;
 extern int _orc_compiler_flag_debug;
 
 void orc_code_chunk_free (OrcCodeChunk *chunk);
 
 #endif
 
+ORC_END_DECLS
+
 #endif
 
