@@ -29,6 +29,8 @@ int n_programs;
 OrcProgram **programs;
 
 int use_inline = FALSE;
+int use_code = FALSE;
+int use_lazy_init = FALSE;
 
 const char *init_function = NULL;
 
@@ -37,7 +39,7 @@ char *target = "sse";
 #define ORC_VERSION(a,b,c,d) ((a)*1000000 + (b)*10000 + (c)*100 + (d))
 #define REQUIRE(a,b,c,d) do { \
   if (ORC_VERSION((a),(b),(c),(d)) > compat) { \
-    fprintf(stderr, "Feature used that is incompatible with --compat\n"); \
+    fprintf(stderr, "Feature used that is incompatible with --compat in program %s\n", p->name); \
     exit (1); \
   } \
 } while (0)
@@ -71,6 +73,7 @@ void help (void)
   printf("  --inline                Generate inline functions in header\n");
   printf("  --no-inline             Do not generate inline functions in header\n");
   printf("  --init-function FUNCTION  Generate initialization function\n");
+  printf("  --lazy-init             Do Orc compile at function execution\n");
   printf("\n");
 
   exit (0);
@@ -149,6 +152,8 @@ main (int argc, char *argv[])
       } else {
         help();
       }
+    } else if (strcmp(argv[i], "--lazy-init") == 0) {
+      use_lazy_init = TRUE;
     } else if (strncmp(argv[i], "-", 1) == 0) {
       printf("Unknown option: %s\n", argv[i]);
       exit (1);
@@ -167,7 +172,7 @@ main (int argc, char *argv[])
     exit (1);
   }
 
-  if (orc_target_get_by_name (target) == NULL) {
+  if (mode == MODE_ASSEMBLY && orc_target_get_by_name (target) == NULL) {
     printf("Unknown target \"%s\"\n", target);
     exit (1);
   }
@@ -190,6 +195,9 @@ main (int argc, char *argv[])
       exit (1);
     }
   }
+  if (compat >= ORC_VERSION(0,4,11,1)) {
+    use_code = TRUE;
+  }
 
   if (output_file == NULL) {
     switch (mode) {
@@ -222,6 +230,10 @@ main (int argc, char *argv[])
     init_function = orc_parse_get_init_function (programs[0]);
   }
 
+  if (init_function == NULL) {
+    use_lazy_init = TRUE;
+  }
+
   output = fopen (output_file, "w");
   if (!output) {
     printf("Could not write output file: %s\n", output_file);
@@ -236,15 +248,15 @@ main (int argc, char *argv[])
     fprintf(output, "#ifdef HAVE_CONFIG_H\n");
     fprintf(output, "#include \"config.h\"\n");
     fprintf(output, "#endif\n");
-    fprintf(output, "#ifndef DISABLE_ORC\n");
-    fprintf(output, "#include <orc/orc.h>\n");
-    fprintf(output, "#endif\n");
     if (include_file) {
       fprintf(output, "#include <%s>\n", include_file);
     }
     fprintf(output, "\n");
     fprintf(output, "%s", orc_target_c_get_typedefs ());
     fprintf(output, "\n");
+    fprintf(output, "#ifndef DISABLE_ORC\n");
+    fprintf(output, "#include <orc/orc.h>\n");
+    fprintf(output, "#endif\n");
     for(i=0;i<n;i++){
       output_code_header (programs[i], output);
     }
@@ -305,8 +317,6 @@ main (int argc, char *argv[])
     fprintf(output, "#endif\n");
     fprintf(output, "\n");
   } else if (mode == MODE_TEST) {
-    fprintf(output, "#include <orc/orc.h>\n");
-    fprintf(output, "#include <orc-test/orctest.h>\n");
     fprintf(output, "#include <stdio.h>\n");
     fprintf(output, "#include <string.h>\n");
     fprintf(output, "#include <stdlib.h>\n");
@@ -316,6 +326,8 @@ main (int argc, char *argv[])
     }
     fprintf(output, "\n");
     fprintf(output, "%s", orc_target_c_get_typedefs ());
+    fprintf(output, "#include <orc/orc.h>\n");
+    fprintf(output, "#include <orc-test/orctest.h>\n");
     fprintf(output, "%s", orc_target_get_asm_preamble ("c"));
     fprintf(output, "\n");
     for(i=0;i<n;i++){
@@ -494,10 +506,10 @@ output_prototype (OrcProgram *p, FILE *output)
     if (var->size) {
       if (need_comma) fprintf(output, ", ");
       if (var->type_name) {
-        fprintf(output, "%s * %s", var->type_name,
+        fprintf(output, "%s * ORC_RESTRICT %s", var->type_name,
             varnames[ORC_VAR_D1 + i]);
       } else {
-        fprintf(output, "orc_uint%d * %s", var->size*8,
+        fprintf(output, "orc_uint%d * ORC_RESTRICT %s", var->size*8,
             varnames[ORC_VAR_D1 + i]);
       }
       if (p->is_2d) {
@@ -511,10 +523,10 @@ output_prototype (OrcProgram *p, FILE *output)
     if (var->size) {
       if (need_comma) fprintf(output, ", ");
       if (var->type_name) {
-        fprintf(output, "%s * %s", var->type_name,
+        fprintf(output, "%s * ORC_RESTRICT %s", var->type_name,
             varnames[ORC_VAR_A1 + i]);
       } else {
-        fprintf(output, "orc_uint%d * %s", var->size*8,
+        fprintf(output, "orc_uint%d * ORC_RESTRICT %s", var->size*8,
             varnames[ORC_VAR_A1 + i]);
       }
       need_comma = TRUE;
@@ -525,10 +537,10 @@ output_prototype (OrcProgram *p, FILE *output)
     if (var->size) {
       if (need_comma) fprintf(output, ", ");
       if (var->type_name) {
-        fprintf(output, "const %s * %s", var->type_name,
+        fprintf(output, "const %s * ORC_RESTRICT %s", var->type_name,
             varnames[ORC_VAR_S1 + i]);
       } else {
-        fprintf(output, "const orc_uint%d * %s", var->size*8,
+        fprintf(output, "const orc_uint%d * ORC_RESTRICT %s", var->size*8,
             varnames[ORC_VAR_S1 + i]);
       }
       if (p->is_2d) {
@@ -655,16 +667,22 @@ output_code_execute (OrcProgram *p, FILE *output, int is_inline)
   OrcVariable *var;
   int i;
 
-  if (init_function) {
+  if (!use_lazy_init) {
+    const char *storage;
     if (is_inline) {
-      fprintf(output, "extern OrcProgram *_orc_program_%s;\n", p->name);
+      storage = "extern ";
     } else {
       if (use_inline) {
-        fprintf(output, "OrcProgram *_orc_program_%s;\n", p->name);
+        storage = "";
       } else {
-        fprintf(output, "static OrcProgram *_orc_program_%s;\n", p->name);
+        storage = "static ";
       }
     }
+    if (use_code) {
+      fprintf(output, "%sOrcCode *_orc_code_%s;\n", storage, p->name);
+    } else {
+      fprintf(output, "%sOrcProgram *_orc_program_%s;\n", storage, p->name);
+    }
   }
   if (is_inline) {
     fprintf(output, "static inline void\n");
@@ -675,29 +693,52 @@ output_code_execute (OrcProgram *p, FILE *output, int is_inline)
   fprintf(output, "\n");
   fprintf(output, "{\n");
   fprintf(output, "  OrcExecutor _ex, *ex = &_ex;\n");
-  if (init_function) {
-    fprintf(output, "  OrcProgram *p = _orc_program_%s;\n", p->name);
+  if (!use_lazy_init) {
+    if (use_code) {
+      fprintf(output, "  OrcCode *c = _orc_code_%s;\n", p->name);
+    } else {
+      if (use_code) {
+        fprintf(output, "  OrcCode *c = _orc_code_%s;\n", p->name);
+      } else {
+        fprintf(output, "  OrcProgram *p = _orc_program_%s;\n", p->name);
+      }
+    }
   } else {
     fprintf(output, "  static int p_inited = 0;\n");
-    fprintf(output, "  static OrcProgram *p = 0;\n");
+    if (use_code) {
+      fprintf(output, "  static OrcCode *c = 0;\n");
+    } else {
+      fprintf(output, "  static OrcProgram *p = 0;\n");
+    }
   }
   fprintf(output, "  void (*func) (OrcExecutor *);\n");
   fprintf(output, "\n");
-  if (init_function == NULL) {
+  if (use_lazy_init) {
     fprintf(output, "  if (!p_inited) {\n");
     fprintf(output, "    orc_once_mutex_lock ();\n");
     fprintf(output, "    if (!p_inited) {\n");
-    fprintf(output, "      OrcCompileResult result;\n");
+    if (use_code) {
+      fprintf(output, "      OrcProgram *p;\n");
+    }
     fprintf(output, "\n");
     output_program_generation (p, output, is_inline);
     fprintf(output, "\n");
-    fprintf(output, "      result = orc_program_compile (p);\n");
+    fprintf(output, "      orc_program_compile (p);\n");
+    if (use_code) {
+      fprintf(output, "      c = orc_program_take_code (p);\n");
+      fprintf(output, "      orc_program_free (p);\n");
+    }
     fprintf(output, "    }\n");
     fprintf(output, "    p_inited = TRUE;\n");
     fprintf(output, "    orc_once_mutex_unlock ();\n");
     fprintf(output, "  }\n");
   }
-  fprintf(output, "  ex->program = p;\n");
+  if (use_code) {
+    fprintf(output, "  ex->arrays[ORC_VAR_A2] = c;\n");
+    fprintf(output, "  ex->program = 0;\n");
+  } else {
+    fprintf(output, "  ex->program = p;\n");
+  }
   fprintf(output, "\n");
   if (p->constant_n) {
     fprintf(output, "  ex->n = %d;\n", p->constant_n);
@@ -778,7 +819,11 @@ output_code_execute (OrcProgram *p, FILE *output, int is_inline)
     }
   }
   fprintf(output, "\n");
-  fprintf(output, "  func = p->code_exec;\n");
+  if (use_code) {
+    fprintf(output, "  func = c->exec;\n");
+  } else {
+    fprintf(output, "  func = p->code_exec;\n");
+  }
   fprintf(output, "  func (ex);\n");
   for(i=0;i<4;i++){
     var = &p->vars[ORC_VAR_A1 + i];
@@ -937,21 +982,28 @@ output_init_function (FILE *output)
   fprintf(output, "void\n");
   fprintf(output, "%s (void)\n", init_function);
   fprintf(output, "{\n");
-  fprintf(output, "#ifndef DISABLE_ORC\n");
-  for(i=0;i<n_programs;i++){
-    fprintf(output, "  {\n");
-    fprintf(output, "    /* %s */\n", programs[i]->name);
-    fprintf(output, "    OrcProgram *p;\n");
-    fprintf(output, "    OrcCompileResult result;\n");
-    fprintf(output, "    \n");
-    output_program_generation (programs[i], output, FALSE);
-    fprintf(output, "\n");
-    fprintf(output, "      result = orc_program_compile (p);\n");
-    fprintf(output, "\n");
-    fprintf(output, "    _orc_program_%s = p;\n", programs[i]->name);
-    fprintf(output, "  }\n");
+  if (!use_lazy_init) {
+    fprintf(output, "#ifndef DISABLE_ORC\n");
+    for(i=0;i<n_programs;i++){
+      fprintf(output, "  {\n");
+      fprintf(output, "    /* %s */\n", programs[i]->name);
+      fprintf(output, "    OrcProgram *p;\n");
+      fprintf(output, "    \n");
+      output_program_generation (programs[i], output, FALSE);
+      fprintf(output, "\n");
+      fprintf(output, "      orc_program_compile (p);\n");
+      fprintf(output, "\n");
+      if (use_code) {
+        fprintf(output, "    _orc_code_%s = orc_program_take_code (p);\n",
+            programs[i]->name);
+        fprintf(output, "    orc_program_free (p);\n");
+      } else {
+        fprintf(output, "    _orc_program_%s = p;\n", programs[i]->name);
+      }
+      fprintf(output, "  }\n");
+    }
+    fprintf(output, "#endif\n");
   }
-  fprintf(output, "#endif\n");
   fprintf(output, "}\n");
   fprintf(output, "\n");
 }
@@ -1118,11 +1170,13 @@ output_code_test (OrcProgram *p, FILE *output)
   fprintf(output, "      printf (\"    backup function  :   PASSED\\n\");\n");
   fprintf(output, "    }\n");
   fprintf(output, "\n");
-  fprintf(output, "    if (benchmark) {\n");
-  fprintf(output, "      printf (\"    cycles (backup)  :   %%g\\n\",\n");
-  fprintf(output, "          orc_test_performance_full (p, ORC_TEST_FLAGS_BACKUP, NULL));\n");
-  fprintf(output, "    }\n");
-  fprintf(output, "\n");
+  if (compat >= ORC_VERSION(0,4,7,1)) {
+    fprintf(output, "    if (benchmark) {\n");
+    fprintf(output, "      printf (\"    cycles (backup)  :   %%g\\n\",\n");
+    fprintf(output, "          orc_test_performance_full (p, ORC_TEST_FLAGS_BACKUP, NULL));\n");
+    fprintf(output, "    }\n");
+    fprintf(output, "\n");
+  }
   fprintf(output, "    ret = orc_test_compare_output (p);\n");
   fprintf(output, "    if (ret == ORC_TEST_INDETERMINATE && !quiet) {\n");
   fprintf(output, "      printf (\"    compiled function:   COMPILE FAILED\\n\");\n");
@@ -1132,10 +1186,12 @@ output_code_test (OrcProgram *p, FILE *output)
   fprintf(output, "      printf (\"    compiled function:   PASSED\\n\");\n");
   fprintf(output, "    }\n");
   fprintf(output, "\n");
-  fprintf(output, "    if (benchmark) {\n");
-  fprintf(output, "      printf (\"    cycles (compiled):   %%g\\n\",\n");
-  fprintf(output, "          orc_test_performance_full (p, 0, NULL));\n");
-  fprintf(output, "    }\n");
+  if (compat >= ORC_VERSION(0,4,7,1)) {
+    fprintf(output, "    if (benchmark) {\n");
+    fprintf(output, "      printf (\"    cycles (compiled):   %%g\\n\",\n");
+    fprintf(output, "          orc_test_performance_full (p, 0, NULL));\n");
+    fprintf(output, "    }\n");
+  }
   fprintf(output, "\n");
   fprintf(output, "    orc_program_free (p);\n");
   fprintf(output, "  }\n");
@@ -1153,7 +1209,7 @@ output_code_assembly (OrcProgram *p, FILE *output)
     OrcCompileResult result;
     OrcTarget *t = orc_target_get_by_name(target);
 
-    result = orc_program_compile_full (p, orc_target_get_by_name(target),
+    result = orc_program_compile_full (p, t,
         orc_target_get_default_flags (t));
     if (ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
       fprintf(output, "%s\n", orc_program_get_asm_code (p));
